/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SynchronizedRulePackage.as 4876 2009-08-11 14:22:08Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.common.model.domain
{

import flash.events.EventDispatcher;

import mx.events.PropertyChangeEvent;
import mx.events.PropertyChangeEventKind;

import org.ow2.jasmine.monitoring.eos.common.utils.logger.EoSLogger;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.IRulePackageVO;


/**
* A container for a rule package to allow an easier synchronisation with
* the server. The container watches the hosted rule package and adjusts its
* state in consequence.
*
* @author Julien Nicoulaud
*/
[Event(name="propertyChange", type="mx.events.PropertyChangeEvent")]
public class SynchronizedRulePackage extends EventDispatcher
{
		
	// =========================================================================
	// Variables
	// =========================================================================

	/**
	* The synch state of the object
    */
	private var _state : SynchronizedRulePackageState;
	
	/**
	* The hosted rule package
	*/
	private var _rulePackage : IRulePackageVO;
	
	
    // =========================================================================
    // Constructor
    // =========================================================================
	
    /**
    * Build a new SynchronizedRulePackage with the given state
    */
    public function SynchronizedRulePackage(rulePackage:IRulePackageVO,
                                            state:SynchronizedRulePackageState=null)
    {
        // Assign variables
        this.rulePackage = rulePackage;
        
        // Set the state
    	if (state!=null) {
    		this.state = state;
    	} else {
            // Set the state as modified by default
            this.state = SynchronizedRulePackageState.MODIFIED;
        }

    }
    
    
    // =========================================================================
    // Getters and setters
    // =========================================================================    
	
	/**
    * Get the current state
	*/
	public function set state(value:SynchronizedRulePackageState):void
	{
	    // Store the old value
	    var oldValue : SynchronizedRulePackageState = state;
	    
	    // Assign the value
	    _state = value;
        
        // Dispatch a property change event
        var event:PropertyChangeEvent = new PropertyChangeEvent(PropertyChangeEvent.PROPERTY_CHANGE,
                                                                true,
                                                                false,
                                                                PropertyChangeEventKind.UPDATE,
                                                                state,
                                                                oldValue,
                                                                value,
                                                                this);
        dispatchEvent(event);
	}
	
	/**
	* Set the current state
	*/
	[Bindable]
	public function get state():SynchronizedRulePackageState
	{
	    return _state;
	}
    
    /**
    * Set the hosted rule package
    */
    public function set rulePackage(value:IRulePackageVO):void
    {
        // Store the old value
        var oldValue : IRulePackageVO = rulePackage;
        
        // Assign the value
        _rulePackage = value;
        
        // Adjust the state
        fieldModifiedHandler();
        
        // Starting listening to modifications
        this.rulePackage.addEventListener(PropertyChangeEvent.PROPERTY_CHANGE,fieldModifiedHandler);
        
        // Dispatch a property change event
        var event:PropertyChangeEvent = new PropertyChangeEvent(PropertyChangeEvent.PROPERTY_CHANGE,
                                                                true,
                                                                false,
                                                                PropertyChangeEventKind.UPDATE,
                                                                rulePackage,
                                                                oldValue,
                                                                value,
                                                                this);
        dispatchEvent(event);
    }
    
    /**
    * Get the hosted rule package
    */
    [Bindable]
    public function get rulePackage():IRulePackageVO
    {
        return _rulePackage;
    }
	
	
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Returns a String preview of the object
    */
    override public function toString():String
    {
    	return "{state:" + state.toString() +
               ",rulePackage:" + "TODO" + "}";
    	       
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
      
    /**
    * Handles observed object changes to adjust the state
    */
    private function fieldModifiedHandler(event:PropertyChangeEvent=null):void
    {
        if (event !== null) {
            
            // If this is a transient property, do nothing
            switch (event.property) {
                case 'nameIsValid':
                case 'nameIsAvailable':
                case 'compilationErrors':
                case 'compilationChecked':
                    return;
                    break;
            }
            
            // If this is not a metadata property, mark the compilation as unchecked
            switch (event.property) {
                case 'displayName':
                case 'systemName':
                case 'previousSystemName':
                case 'description':
                case 'state':
                case 'ruleEngineInstanceName':
                    break;
                default:
                    rulePackage.compilationChecked = false;
                    break;
            }
        }
        
        // Adjust the state of the synchronized object
        switch(state) {
        	case SynchronizedRulePackageState.SYNCHRONIZED:
        	   state = SynchronizedRulePackageState.MODIFIED;
        	   break;
            case SynchronizedRulePackageState.OBSOLETE:
               state = SynchronizedRulePackageState.OUT_OF_SYNC;
               break;
        }
    }

}
}
